/*
 * =====================================================================================
 * 
 *     Modulename:
 *       Filename:  watchface_chinese.c
 *
 *    Description:  数字运动目标表盘
 *    Corporation:
 * 
 *         Author:  稻米鼠 ,i@zji.me 
 *        Created:  2016年9月19日 09:27:12
 *
 * =====================================================================================
 */	
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "maibu_sdk.h"
#include "maibu_res.h"

#define TIME_NUM 4 //时间位数
#define SPORT_NUM 6 //时间位数

/*图片坐标：依次为时间的1~4位*/
static uint8_t ORIGIN_X[4] = {25, 43, 71, 89};
static uint8_t ORIGIN_Y[4] = {48, 48, 48, 48};

static uint8_t ORIGINS_X[6] = {35, 45, 55, 65, 75, 85};
static uint8_t ORIGINS_Y[6] = {96, 96, 96, 96, 96, 96};

/*图片名数组：第一项为背景图片，后面依次为数字0~9图片*/
uint16_t bmp_array[10] = {PIC_0_1, PIC_1_1, PIC_2_1, PIC_3_1, PIC_4_1, PIC_5_1, PIC_6_1, PIC_7_1, PIC_8_1, PIC_9_1};
uint16_t bmps_array[10] = {PIC_0_2, PIC_1_2, PIC_2_2, PIC_3_2, PIC_4_2, PIC_5_2, PIC_6_2, PIC_7_2, PIC_8_2, PIC_9_2};


/*小时分钟月日图层ID，通过该图层ID获取图层句柄*/
static int8_t g_layer_time[TIME_NUM] ={ -1, -1, -1, -1};
static int8_t g_layer_sport[SPORT_NUM] ={ -1, -1, -1, -1, -1, -1};
static int8_t g_layer_c = -1;
static int8_t g_c_x = 0;
static int8_t g_c_y = 0;
/*窗口ID, 通过该窗口ID获取窗口句柄*/
static int32_t g_window = -1;

/*
 *--------------------------------------------------------------------------------------
 *     function:  get_bg_layer
 *    parameter: 
 *       return:
 *  description:  
 * 	  other:
 *--------------------------------------------------------------------------------------
 */
static P_Layer get_bg_layer()
{
	/*创建位图图层显示框架范围，起始点{x=32,y=32},屏幕左上角为{0,0},高度宽度{h=64,w=64},位图大小必须小于等于框架*/
	GRect frame = {{0, 0}, {128, 128}};
	GBitmap bitmap;	
	/*获取位图资源, 宏RES_BITMAP_WATCHFACE_WEATHER_CLOUDY由用户在appinfo.json中配置*/
	res_get_user_bitmap(PIC_background, &bitmap);
	/*生成位图结构体, 依次为位图资源、显示位置、对齐方式*/
	LayerBitmap layer_bg = {bitmap, frame, GAlignCenter};
	/*创建图层*/
	P_Layer layerbg = app_layer_create_bitmap(&layer_bg);
	app_layer_set_bg_color(layerbg, GColorWhite);

	return (layerbg);
}

/*
 *--------------------------------------------------------------------------------------
 *     function:  app_watch_update
 *    parameter: 
 *       return:
 *  description:  更新时间图层
 * 	      other:
 *--------------------------------------------------------------------------------------
 */
static void app_watch_update()
{
	P_Window p_window = NULL;
	P_Layer p_layer = NULL, g_layer_c_n = NULL;
	GBitmap bitmap;
	struct date_time datetime;
	app_service_get_datetime(&datetime);
	uint8_t new_time[TIME_NUM] = {datetime.hour/10, datetime.hour%10, datetime.min/10, datetime.min%10};
	uint8_t pos;
	
    SportData data;
	maibu_get_sport_data(&data, 0);
    uint8_t new_sport[SPORT_NUM] = {data.step/100000, (data.step%100000)/10000, ((data.step%100000)%10000)/1000, (((data.step%100000)%10000)%1000)/100, ((((data.step%100000)%10000)%1000)%100)/10, ((((data.step%100000)%10000)%1000)%100)%10};
	
	/*根据窗口ID获取窗口句柄*/
	p_window = app_window_stack_get_window_by_id(g_window);
	if (p_window == NULL)
	{
		return;
	}
	for (pos=0; pos<TIME_NUM; pos++)
	{
			/*获取时间图层句柄*/
			p_layer = app_window_get_layer_by_id(p_window, g_layer_time[pos]);	
			if (p_layer != NULL)
			{
				/*更新时间图层图片*/
					if (pos < 2)
					{
						res_get_user_bitmap(bmp_array[new_time[pos]], &bitmap);
					}else
					{
						res_get_user_bitmap(bmp_array[new_time[pos]], &bitmap);
					}
				app_layer_set_bitmap_bitmap(p_layer, &bitmap);
			}
	}
    for (pos=0; pos<SPORT_NUM; pos++)
	{
		p_layer = app_window_get_layer_by_id(p_window, g_layer_sport[pos]);	
			if (p_layer != NULL)
			{
				/*更新运动图层图片*/
				res_get_user_bitmap(bmps_array[new_sport[pos]], &bitmap);
				app_layer_set_bitmap_bitmap(p_layer, &bitmap);
			}
	}

    p_layer = app_window_get_layer_by_id(p_window, g_layer_c);
    Geometry *geometry[2]; /*创建存储几何图形的指针数组, 几何图形数量不能超过指针数组大小*/
	uint8_t num = 0;
	LayerGeometry layer_geometry;	/*几何图层结构体*/
	memset(geometry, 0, sizeof(geometry));
    /* 度数计算 225-270*data.step/10000 */
    g_c_x = 64+60*cos((225-270*data.step/10000)*3.1415/180);
    g_c_y = 64-60*sin((225-270*data.step/10000)*3.1415/180);
    Circle c = {{g_c_x, g_c_y}, 4};
    Geometry lg = {GeometryTypeCircle, FillArea, GColorBlack, (void*)&c};
    geometry[num++] = &lg;
    Circle d = {{g_c_x, g_c_y}, 3};
    Geometry ly = {GeometryTypeCircle, FillArea, GColorWhite, (void*)&d};
    geometry[num++] = &ly;

    layer_geometry.num = num;
	layer_geometry.p_g = geometry;

	/*创建几何图层*/
	g_layer_c_n = app_layer_create_geometry(&layer_geometry);
	g_layer_c = app_window_replace_layer(p_window, p_layer, g_layer_c_n);

	/*窗口显示*/	
	app_window_update(p_window);
}


/*
 *--------------------------------------------------------------------------------------
 *     function:  app_watch_time_change 
 *    parameter: 
 *       return:
 *  description:  系统时间有变化时，更新时间图层
 * 	      other:
 *--------------------------------------------------------------------------------------
 */
static void app_watch_time_change(enum SysEventType type, void *context)
{

	/*时间更改*/
	if (type == SysEventTypeTimeChange)
	{
		app_watch_update();	
	}
}



/*
 *--------------------------------------------------------------------------------------
 *     function:  get_layer
 *    parameter: 
 *       return:
 *  description:  生成表盘窗口的各图层
 * 	      other:
 *--------------------------------------------------------------------------------------
 */
static P_Layer get_layer(uint8_t pos, uint8_t time[])
{
	GRect frame = {{ORIGIN_X[pos], ORIGIN_Y[pos]}, {32, 16}};
	GBitmap bitmap;
		
	/*获取用户图片*/
	if (pos < 2)
	{
		res_get_user_bitmap(bmp_array[time[pos]], &bitmap);
	}else
	{
		res_get_user_bitmap(bmp_array[time[pos]], &bitmap);
	}

	LayerBitmap layerbmp = { bitmap, frame, GAlignCenter};	

	/*生成图层*/
	P_Layer	 layer_back = NULL;
	layer_back = app_layer_create_bitmap(&layerbmp);

	return (layer_back);
}
static P_Layer get_sport_layer(uint8_t pos, uint8_t sport[])
{
	GRect frame = {{ORIGINS_X[pos], ORIGINS_Y[pos]}, {16, 8}};
	GBitmap bitmap;
		
	/*获取用户图片*/
	res_get_user_bitmap(bmps_array[sport[pos]], &bitmap);

	LayerBitmap layerbmp = { bitmap, frame, GAlignCenter};	

	/*生成图层*/
	P_Layer	 layer_back = NULL;
	layer_back = app_layer_create_bitmap(&layerbmp);

	return (layer_back);
}

/*
 *--------------------------------------------------------------------------------------
 *     function:  init_watch
 *    parameter: 
 *       return:
 *  description:  生成表盘窗口
 * 	      other:
 *--------------------------------------------------------------------------------------
 */
static P_Window init_watch(void)
{
	P_Window p_window = NULL;
	P_Layer layer_background = NULL, layer_time = NULL, layer_sport = NULL, layer_c = NULL;
	uint8_t pos;
	struct date_time datetime;
	app_service_get_datetime(&datetime);
	uint8_t current_time[TIME_NUM] = {datetime.hour/10, datetime.hour%10, datetime.min/10, datetime.min%10};
    SportData data;
	maibu_get_sport_data(&data, 0);
    uint8_t current_sport[SPORT_NUM] = {data.step/100000, (data.step%100000)/10000, ((data.step%100000)%10000)/1000, (((data.step%100000)%10000)%1000)/100, ((((data.step%100000)%10000)%1000)%100)/10, ((((data.step%100000)%10000)%1000)%100)%10};
	/*创建一个窗口*/
	p_window = app_window_create();
	if (NULL == p_window)
	{
		return NULL;
	}
    layer_background = get_bg_layer();
    app_window_add_layer(p_window, layer_background);
	/*创建时间图层并添加到窗口*/
	for (pos=0; pos<TIME_NUM; pos++)
	{
		layer_time = get_layer(pos, current_time);
		g_layer_time[pos] = app_window_add_layer(p_window, layer_time);
	}

    /*创建运动量图层并添加到窗口*/
	for (pos=0; pos<SPORT_NUM; pos++)
	{
		layer_sport = get_sport_layer(pos, current_sport);
		g_layer_sport[pos] = app_window_add_layer(p_window, layer_sport);
	}

    Geometry *geometry[2]; /*创建存储几何图形的指针数组, 几何图形数量不能超过指针数组大小*/
	uint8_t num = 0;
	LayerGeometry layer_geometry;	/*几何图层结构体*/
	memset(geometry, 0, sizeof(geometry));
    /* 度数计算 225-270*data.step/10000 */
    g_c_x = 64+60*cos((225-270*data.step/10000)*3.1415/180);
    g_c_y = 64-60*sin((225-270*data.step/10000)*3.1415/180);
    Circle c = {{g_c_x, g_c_y}, 4};
    Geometry lg = {GeometryTypeCircle, FillArea, GColorBlack, (void*)&c};
    geometry[num++] = &lg;
    Circle d = {{g_c_x, g_c_y}, 3};
    Geometry ly = {GeometryTypeCircle, FillArea, GColorWhite, (void*)&d};
    geometry[num++] = &ly;

    layer_geometry.num = num;
	layer_geometry.p_g = geometry;

	/*创建几何图层*/
	layer_c = app_layer_create_geometry(&layer_geometry);
	g_layer_c = app_window_add_layer(p_window, layer_c);
	

	/*注册一个事件通知回调，当有时间改变时，立即更新时间*/
	maibu_service_sys_event_subscribe(app_watch_time_change);

	return p_window;
}



/*
 *--------------------------------------------------------------------------------------
 *     function:  main
 *    parameter: 
 *       return:
 *  description:  主程序
 * 	      other:
 *--------------------------------------------------------------------------------------
 */
int main()
{
	/*模拟器模拟显示时打开，打包时屏蔽*/
	// simulator_init();

	/*创建显示表盘窗口*/
	P_Window p_window = init_watch();
	if (p_window != NULL)
	{
		/*放入窗口栈显示*/
		g_window = app_window_stack_push(p_window);
	}
	
	/*模拟器模拟显示时打开，打包时屏蔽*/
	// simulator_wait();

	return 0;
}

